package io.github.paulgriffith.main

import com.formdev.flatlaf.extras.components.FlatTabbedPane
import io.github.paulgriffith.MainPanel
import io.github.paulgriffith.utils.Action
import io.github.paulgriffith.utils.ToolPanel
import io.github.paulgriffith.utils.attachPopupMenu
import io.github.paulgriffith.utils.truncate
import java.awt.Dimension
import javax.swing.JFrame
import javax.swing.JMenu
import javax.swing.JMenuBar
import javax.swing.JPopupMenu
import kotlin.io.path.name

class TabPanel : FlatTabbedPane() {
    init {
        tabLayoutPolicy = SCROLL_TAB_LAYOUT
        isTabsClosable = true
        setTabCloseCallback { _, i ->
            removeTabAt(i)
        }

        attachPopupMenu { event ->
            val tabIndex = indexAtLocation(event.x, event.y)
            if (tabIndex == -1) return@attachPopupMenu null
            val tab = getComponentAt(tabIndex)
            if (tab is ToolPanel) {
                JPopupMenu().apply {
                    add(
                        Action(name = "Float") {
                            val frame = createPopupFrame(tab)
                            frame.isVisible = true
                        }
                    )
                    tab.customizePopupMenu(this)
                }
            } else {
                null
            }
        }
    }

    private fun createPopupFrame(tab: ToolPanel): JFrame = JFrame(tab.path.name).apply {
        preferredSize = Dimension(1024, 768)
        iconImage = MainPanel.FRAME_ICON
        defaultCloseOperation = JFrame.DISPOSE_ON_CLOSE
        add(tab)
        val menuBar = JMenuBar()
        menuBar.add(
            JMenu("Actions").apply {
                add(
                    Action(name = "Unfloat") {
                        addTab(
                            tab.path.name.truncate(),
                            tab.icon,
                            tab,
                            tab.path.toString(),
                        )
                        dispose()
                    }
                )
            }
        )

        jMenuBar = menuBar
        pack()
    }
}
